@@ -88,6 +88,16 @@ parse_form(Dir, File, Form, Used) ->
 	 [_, {string, Line, Str}]
 	} ->
 	    process_string(Dir, File, Line, Str, Used);
+	{call,
+	 _,
+	 {remote, _, {atom, _, translate}, {atom, _, translate}},
+	 [_,
+	  {bin,_,
+	   [{bin_element,_,
+	     {string,Line,Str},
+	     default,default}]}]
+	} ->
+	    process_string(Dir, File, Line, Str, Used);
 	{call,
 	 _,
 	 {remote, _, {atom, _, translate}, {atom, _, translate}},
@@ -281,14 +291,14 @@ build_additional_translators(List) ->
       List).
 
 print_translation(File, Line, Str, StrT) ->
-    {ok, StrQ, _} = regexp:gsub(Str, "\"", "\\\""),
-    {ok, StrTQ, _} = regexp:gsub(StrT, "\"", "\\\""),
+    StrQ = ejabberd_regexp:greplace(list_to_binary(Str), <<"\\\"">>, <<"\\\\\"">>),
+    StrTQ = ejabberd_regexp:greplace(list_to_binary(StrT), <<"\\\"">>, <<"\\\\\"">>),
     io:format("#: ~s:~p~nmsgid \"~s\"~nmsgstr \"~s\"~n~n", [File, Line, StrQ, StrTQ]).
 
 print_translation_obsolete(Str, StrT) ->
     File = "unknown.erl",
     Line = 1,
-    {ok, StrQ, _} = regexp:gsub(Str, "\"", "\\\""),
-    {ok, StrTQ, _} = regexp:gsub(StrT, "\"", "\\\""),
+    StrQ = ejabberd_regexp:greplace(Str, "\\\"", "\\\\\""),
+    StrTQ = ejabberd_regexp:greplace(StrT, "\\\"", "\\\\\""),
     io:format("#: ~s:~p~n#~~ msgid \"~s\"~n#~~ msgstr \"~s\"~n~n", [File, Line, StrQ, StrTQ]).
 
