@@ -5,7 +5,7 @@
 %%% Created : 31 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2011   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2014   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
 %%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 %%% General Public License for more details.
 %%%
-%%% You should have received a copy of the GNU General Public License
-%%% along with this program; if not, write to the Free Software
-%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-%%% 02111-1307 USA
+%%% You should have received a copy of the GNU General Public License along
+%%% with this program; if not, write to the Free Software Foundation, Inc.,
+%%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
 %%%----------------------------------------------------------------------
 
@@ -29,38 +28,36 @@
 
 -behaviour(application).
 
--export([start_modules/0,start/2, get_log_path/0, prep_stop/1, stop/1, init/0]).
+-export([start_modules/0,start/2, prep_stop/1, stop/1, init/0]).
 
 -include("ejabberd.hrl").
-
+-include("logger.hrl").
 
 %%%
 %%% Application API
 %%%
 
 start(normal, _Args) ->
-    ejabberd_loglevel:set(4),
+    ejabberd_logger:start(),
     write_pid_file(),
-    application:start(sasl),
+    start_apps(),
+    ejabberd:check_app(ejabberd),
     randoms:start(),
     db_init(),
-    sha:start(),
-    stringprep_sup:start_link(),
-    xml:start(),
     start(),
     translate:start(),
-    acl:start(),
     ejabberd_ctl:init(),
     ejabberd_commands:init(),
     ejabberd_admin:start(),
     gen_mod:start(),
     ejabberd_config:start(),
-    ejabberd_check:config(),
+    set_loglevel_from_config(),
+    acl:start(),
+    shaper:start(),
     connect_nodes(),
-    %% Loading ASN.1 driver explicitly to avoid races in LDAP
-    catch asn1rt:load_driver(),
     Sup = ejabberd_sup:start_link(),
     ejabberd_rdbms:start(),
+    ejabberd_riak_sup:start(),
     ejabberd_auth:start(),
     cyrsasl:start(),
     % Profiling
@@ -78,6 +75,7 @@ start(_, _) ->
 %% This function is called when an application is about to be stopped,
 %% before shutting down the processes of the application.
 prep_stop(State) ->
+    ejabberd_listener:stop_listeners(),
     stop_modules(),
     ejabberd_admin:stop(),
     broadcast_c2s_shutdown(),
@@ -101,94 +99,82 @@ start() ->
 
 init() ->
     register(ejabberd, self()),
-    %erlang:system_flag(fullsweep_after, 0),
-    %error_logger:logfile({open, ?LOG_PATH}),
-    LogPath = get_log_path(),
-    error_logger:add_report_handler(ejabberd_logger_h, LogPath),
-    erl_ddll:load_driver(ejabberd:get_so_path(), tls_drv),
-    case erl_ddll:load_driver(ejabberd:get_so_path(), expat_erl) of
-	ok -> ok;
-	{error, already_loaded} -> ok
-    end,
-    Port = open_port({spawn, "expat_erl"}, [binary]),
-    loop(Port).
+    loop().
 
-
-loop(Port) ->
+loop() ->
     receive
 	_ ->
-	    loop(Port)
+	    loop()
     end.
 
 db_init() ->
+    MyNode = node(),
+    DbNodes = mnesia:system_info(db_nodes),
+    case lists:member(MyNode, DbNodes) of
+	true ->
+	    ok;
+	false ->
+	    ?CRITICAL_MSG("Node name mismatch: I'm [~s], "
+			  "the database is owned by ~p", [MyNode, DbNodes]),
+	    ?CRITICAL_MSG("Either set ERLANG_NODE in ejabberdctl.cfg "
+			  "or change node name in Mnesia", []),
+	    erlang:error(node_name_mismatch)
+    end,
     case mnesia:system_info(extra_db_nodes) of
 	[] ->
 	    mnesia:create_schema([node()]);
 	_ ->
 	    ok
     end,
-    application:start(mnesia, permanent),
+    ejabberd:start_app(mnesia, permanent),
     mnesia:wait_for_tables(mnesia:system_info(local_tables), infinity).
 
 %% Start all the modules in all the hosts
 start_modules() ->
     lists:foreach(
       fun(Host) ->
-	      case ejabberd_config:get_local_option({modules, Host}) of
-		  undefined ->
-		      ok;
-		  Modules ->
-		      lists:foreach(
-			fun({Module, Args}) ->
-				gen_mod:start_module(Host, Module, Args)
-			end, Modules)
-	      end
+              Modules = ejabberd_config:get_option(
+                          {modules, Host},
+                          fun(Mods) ->
+                                  lists:map(
+                                    fun({M, A}) when is_atom(M), is_list(A) ->
+                                            {M, A}
+                                    end, Mods)
+                          end, []),
+              lists:foreach(
+                fun({Module, Args}) ->
+                        gen_mod:start_module(Host, Module, Args)
+                end, Modules)
       end, ?MYHOSTS).
 
 %% Stop all the modules in all the hosts
 stop_modules() ->
     lists:foreach(
       fun(Host) ->
-	      case ejabberd_config:get_local_option({modules, Host}) of
-		  undefined ->
-		      ok;
-		  Modules ->
-		      lists:foreach(
-			fun({Module, _Args}) ->
-				gen_mod:stop_module_keep_config(Host, Module)
-			end, Modules)
-	      end
+              Modules = ejabberd_config:get_option(
+                          {modules, Host},
+                          fun(Mods) ->
+                                  lists:map(
+                                    fun({M, A}) when is_atom(M), is_list(A) ->
+                                            {M, A}
+                                    end, Mods)
+                          end, []),
+              lists:foreach(
+                fun({Module, _Args}) ->
+                        gen_mod:stop_module_keep_config(Host, Module)
+                end, Modules)
       end, ?MYHOSTS).
 
 connect_nodes() ->
-    case ejabberd_config:get_local_option(cluster_nodes) of
-	undefined ->
-	    ok;
-	Nodes when is_list(Nodes) ->
-	    lists:foreach(fun(Node) ->
-				  net_kernel:connect_node(Node)
-			  end, Nodes)
-    end.
-
-%% @spec () -> string()
-%% @doc Returns the full path to the ejabberd log file.
-%% It first checks for application configuration parameter 'log_path'.
-%% If not defined it checks the environment variable EJABBERD_LOG_PATH.
-%% And if that one is neither defined, returns the default value:
-%% "ejabberd.log" in current directory.
-get_log_path() ->
-    case application:get_env(log_path) of
-	{ok, Path} ->
-	    Path;
-	undefined ->
-	    case os:getenv("EJABBERD_LOG_PATH") of
-		false ->
-		    ?LOG_PATH;
-		Path ->
-		    Path
-	    end
-    end.
-
+    Nodes = ejabberd_config:get_option(
+              cluster_nodes,
+              fun(Ns) ->
+                      true = lists:all(fun is_atom/1, Ns),
+                      Ns
+              end, []),
+    lists:foreach(fun(Node) ->
+                          net_kernel:connect_node(Node)
+                  end, Nodes).
 
 %% If ejabberd is running on some Windows machine, get nameservers and add to Erlang
 maybe_add_nameservers() ->
@@ -204,10 +190,12 @@ add_windows_nameservers() ->
 
 
 broadcast_c2s_shutdown() ->
-    Children = supervisor:which_children(ejabberd_c2s_sup),
+    Children = ejabberd_sm:get_all_pids(),
     lists:foreach(
-      fun({_, C2SPid, _, _}) ->
-	      C2SPid ! system_shutdown
+      fun(C2SPid) when node(C2SPid) == node() ->
+	      C2SPid ! system_shutdown;
+	 (_) ->
+	      ok
       end, Children).
 
 %%%
@@ -239,3 +227,20 @@ delete_pid_file() ->
 	PidFilename ->
 	    file:delete(PidFilename)
     end.
+
+set_loglevel_from_config() ->
+    Level = ejabberd_config:get_option(
+              loglevel,
+              fun(P) when P>=0, P=<5 -> P end,
+              4),
+    ejabberd_logger:set(Level).
+
+start_apps() ->
+    ejabberd:start_app(sasl),
+    ejabberd:start_app(ssl),
+    ejabberd:start_app(p1_yaml),
+    ejabberd:start_app(p1_tls),
+    ejabberd:start_app(p1_xml),
+    ejabberd:start_app(p1_stringprep),
+    ejabberd:start_app(p1_zlib),
+    ejabberd:start_app(p1_cache_tab).
