@@ -5,7 +5,7 @@
 %%% Created : 16 Nov 2002 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2011   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2014   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
 %%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 %%% General Public License for more details.
 %%%
-%%% You should have received a copy of the GNU General Public License
-%%% along with this program; if not, write to the Free Software
-%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-%%% 02111-1307 USA
+%%% You should have received a copy of the GNU General Public License along
+%%% with this program; if not, write to the Free Software Foundation, Inc.,
+%%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
 %%%----------------------------------------------------------------------
 
@@ -35,10 +34,13 @@
 	 stop_listener/2,
 	 parse_listener_portip/2,
 	 add_listener/3,
-	 delete_listener/2
+	 delete_listener/2,
+         transform_options/1,
+         validate_cfg/1
 	]).
 
 -include("ejabberd.hrl").
+-include("logger.hrl").
 
 %% We do not block on send anymore.
 -define(TCP_SEND_TIMEOUT, 15000).
@@ -53,7 +55,7 @@ init(_) ->
     {ok, {{one_for_one, 10, 1}, []}}.
 
 bind_tcp_ports() ->
-    case ejabberd_config:get_local_option(listen) of
+    case ejabberd_config:get_option(listen, fun validate_cfg/1) of
 	undefined ->
 	    ignore;
 	Ls ->
@@ -77,7 +79,8 @@ bind_tcp_port(PortIP, Module, RawOpts) ->
 		udp -> ok;
 		_ ->
 		    ListenSocket = listen_tcp(PortIP, Module, SockOpts, Port, IPS),
-		    ets:insert(listen_sockets, {PortIP, ListenSocket})
+		    ets:insert(listen_sockets, {PortIP, ListenSocket}),
+                    ok
 	    end
     catch
 	throw:{error, Error} ->
@@ -85,7 +88,7 @@ bind_tcp_port(PortIP, Module, RawOpts) ->
     end.
 
 start_listeners() ->
-    case ejabberd_config:get_local_option(listen) of
+    case ejabberd_config:get_option(listen, fun validate_cfg/1) of
 	undefined ->
 	    ignore;
 	Ls ->
@@ -148,7 +151,20 @@ init_udp(PortIP, Module, Opts, SockOpts, Port, IPS) ->
 	{ok, Socket} ->
 	    %% Inform my parent that this port was opened succesfully
 	    proc_lib:init_ack({ok, self()}),
-	    udp_recv(Socket, Module, Opts);
+	    case erlang:function_exported(Module, udp_init, 2) of
+		false ->
+		    udp_recv(Socket, Module, Opts);
+		true ->
+		    case catch Module:udp_init(Socket, Opts) of
+			{'EXIT', _} = Err ->
+			    ?ERROR_MSG("failed to process callback function "
+				       "~p:~s(~p, ~p): ~p",
+				       [Module, udp_init, Socket, Opts, Err]),
+			    udp_recv(Socket, Module, Opts);
+			NewOpts ->
+			    udp_recv(Socket, Module, NewOpts)
+		    end
+	    end;
 	{error, Reason} ->
 	    socket_error(Reason, PortIP, Module, SockOpts, Port, IPS)
     end.
@@ -157,8 +173,20 @@ init_tcp(PortIP, Module, Opts, SockOpts, Port, IPS) ->
     ListenSocket = listen_tcp(PortIP, Module, SockOpts, Port, IPS),
     %% Inform my parent that this port was opened succesfully
     proc_lib:init_ack({ok, self()}),
-    %% And now start accepting connection attempts
-    accept(ListenSocket, Module, Opts).
+    case erlang:function_exported(Module, tcp_init, 2) of
+	false ->
+	    accept(ListenSocket, Module, Opts);
+	true ->
+	    case catch Module:tcp_init(ListenSocket, Opts) of
+		{'EXIT', _} = Err ->
+		    ?ERROR_MSG("failed to process callback function "
+			       "~p:~s(~p, ~p): ~p",
+			       [Module, tcp_init, ListenSocket, Opts, Err]),
+		    accept(ListenSocket, Module, Opts);
+		NewOpts ->
+		    accept(ListenSocket, Module, NewOpts)
+	    end
+    end.
 
 listen_tcp(PortIP, Module, SockOpts, Port, IPS) ->
     case ets:lookup(listen_sockets, PortIP) of
@@ -173,7 +201,11 @@ listen_tcp(PortIP, Module, SockOpts, Port, IPS) ->
 			catch
 			    _:_ -> []
 			end,
-	    Res = gen_tcp:listen(Port, [binary,
+	    DeliverAs = case Module of
+			    ejabberd_xmlrpc -> list;
+			    _ -> binary
+			end,
+	    Res = gen_tcp:listen(Port, [DeliverAs,
 					{packet, 0},
 					{active, false},
 					{reuseaddr, true},
@@ -215,17 +247,17 @@ parse_listener_portip(PortIP, Opts) ->
 	case add_proto(PortIP, Opts) of
 	    {P, Prot} ->
 		T = get_ip_tuple(IPOpt, IPVOpt),
-		S = inet_parse:ntoa(T),
+		S = jlib:ip_to_list(T),
 		{P, T, S, Prot};
 	    {P, T, Prot} when is_integer(P) and is_tuple(T) ->
-		S = inet_parse:ntoa(T),
+		S = jlib:ip_to_list(T),
 		{P, T, S, Prot};
-	    {P, S, Prot} when is_integer(P) and is_list(S) ->
-		[S | _] = string:tokens(S, "/"),
-		{ok, T} = inet_parse:address(S),
+	    {P, S, Prot} when is_integer(P) and is_binary(S) ->
+		[S | _] = str:tokens(S, <<"/">>),
+		{ok, T} = inet_parse:address(binary_to_list(S)),
 		{P, T, S, Prot}
 	end,
-    IPV = case size(IPT) of
+    IPV = case tuple_size(IPT) of
 	      4 -> inet;
 	      8 -> inet6
 	  end,
@@ -260,7 +292,7 @@ strip_ip_option(Opts) ->
 			Opts),
     case IPL of
 	%% Only the first ip option is considered
-	[{ip, T1} | _] when is_tuple(T1) ->
+	[{ip, T1} | _] ->
 	    {T1, OptsNoIP};
 	[] ->
 	    {no_ip_option, OptsNoIP}
@@ -277,9 +309,10 @@ accept(ListenSocket, Module, Opts) ->
     case gen_tcp:accept(ListenSocket) of
 	{ok, Socket} ->
 	    case {inet:sockname(Socket), inet:peername(Socket)} of
-		{{ok, Addr}, {ok, PAddr}} ->
-		    ?INFO_MSG("(~w) Accepted connection ~w -> ~w",
-			      [Socket, PAddr, Addr]);
+		{{ok, {Addr, Port}}, {ok, {PAddr, PPort}}} ->
+		    ?INFO_MSG("(~w) Accepted connection ~s:~p -> ~s:~p",
+			      [Socket, inet_parse:ntoa(PAddr), PPort,
+			       inet_parse:ntoa(Addr), Port]);
 		_ ->
 		    ok
 	    end,
@@ -290,8 +323,8 @@ accept(ListenSocket, Module, Opts) ->
 	    CallMod:start(strip_frontend(Module), gen_tcp, Socket, Opts),
 	    accept(ListenSocket, Module, Opts);
 	{error, Reason} ->
-	    ?INFO_MSG("(~w) Failed TCP accept: ~w",
-		      [ListenSocket, Reason]),
+	    ?ERROR_MSG("(~w) Failed TCP accept: ~w",
+                       [ListenSocket, Reason]),
 	    accept(ListenSocket, Module, Opts)
     end.
 
@@ -303,11 +336,11 @@ udp_recv(Socket, Module, Opts) ->
 		    ?ERROR_MSG("failed to process UDP packet:~n"
 			       "** Source: {~p, ~p}~n"
 			       "** Reason: ~p~n** Packet: ~p",
-			       [Addr, Port, Reason, Packet]);
-		_ ->
-		    ok
-	    end,
-	    udp_recv(Socket, Module, Opts);
+			       [Addr, Port, Reason, Packet]),
+		    udp_recv(Socket, Module, Opts);
+		NewOpts ->
+		    udp_recv(Socket, Module, NewOpts)
+	    end;
 	{error, Reason} ->
 	    ?ERROR_MSG("unexpected UDP error: ~s", [format_error(Reason)]),
 	    throw({error, Reason})
@@ -333,11 +366,12 @@ start_listener2(Port, Module, Opts) ->
     %% It is only required to start the supervisor in some cases.
     %% But it doesn't hurt to attempt to start it for any listener.
     %% So, it's normal (and harmless) that in most cases this call returns: {error, {already_started, pid()}}
+    maybe_start_sip(Module),
     start_module_sup(Port, Module),
     start_listener_sup(Port, Module, Opts).
 
 start_module_sup(_Port, Module) ->
-    Proc1 = gen_mod:get_module_proc("sup", Module),
+    Proc1 = gen_mod:get_module_proc(<<"sup">>, Module),
     ChildSpec1 =
 	{Proc1,
 	 {ejabberd_tmp_sup, start_link, [Proc1, strip_frontend(Module)]},
@@ -357,7 +391,7 @@ start_listener_sup(Port, Module, Opts) ->
     supervisor:start_child(ejabberd_listeners, ChildSpec).
 
 stop_listeners() ->
-    Ports = ejabberd_config:get_local_option(listen),
+    Ports = ejabberd_config:get_option(listen, fun validate_cfg/1),
     lists:foreach(
       fun({PortIpNetp, Module, _Opts}) ->
 	      delete_listener(PortIpNetp, Module)
@@ -390,7 +424,8 @@ add_listener(PortIP, Module, Opts) ->
     PortIP1 = {Port, IPT, Proto},
     case start_listener(PortIP1, Module, Opts) of
 	{ok, _Pid} ->
-	    Ports = case ejabberd_config:get_local_option(listen) of
+	    Ports = case ejabberd_config:get_option(
+                           listen, fun validate_cfg/1) of
 			undefined ->
 			    [];
 			Ls ->
@@ -398,7 +433,8 @@ add_listener(PortIP, Module, Opts) ->
 		    end,
 	    Ports1 = lists:keydelete(PortIP1, 1, Ports),
 	    Ports2 = [{PortIP1, Module, Opts} | Ports1],
-	    ejabberd_config:add_local_option(listen, Ports2),
+            Ports3 = lists:map(fun transform_option/1, Ports2),
+	    ejabberd_config:add_option(listen, Ports3),
 	    ok;
 	{error, {already_started, _Pid}} ->
 	    {error, {already_started, PortIP}};
@@ -420,24 +456,35 @@ delete_listener(PortIP, Module) ->
 delete_listener(PortIP, Module, Opts) ->
     {Port, IPT, _, _, Proto, _} = parse_listener_portip(PortIP, Opts),
     PortIP1 = {Port, IPT, Proto},
-    Ports = case ejabberd_config:get_local_option(listen) of
+    Ports = case ejabberd_config:get_option(
+                   listen, fun validate_cfg/1) of
 		undefined ->
 		    [];
 		Ls ->
 		    Ls
 	    end,
     Ports1 = lists:keydelete(PortIP1, 1, Ports),
-    ejabberd_config:add_local_option(listen, Ports1),
+    Ports2 = lists:map(fun transform_option/1, Ports1),
+    ejabberd_config:add_option(listen, Ports2),
     stop_listener(PortIP1, Module).
 
+
+-spec is_frontend({frontend, module} | module()) -> boolean().
+
 is_frontend({frontend, _Module}) -> true;
 is_frontend(_) -> false.
 
 %% @doc(FrontMod) -> atom()
 %% where FrontMod = atom() | {frontend, atom()}
+-spec strip_frontend({frontend, module()} | module()) -> module().
+
 strip_frontend({frontend, Module}) -> Module;
 strip_frontend(Module) when is_atom(Module) -> Module.
 
+maybe_start_sip(esip_socket) ->
+    ejabberd:start_app(esip);
+maybe_start_sip(_) ->
+    ok.
 
 %%%
 %%% Check options
@@ -477,9 +524,10 @@ certfile_readable(Opts) ->
     case proplists:lookup(certfile, Opts) of
 	none -> true;
 	{certfile, Path} ->
-	    case ejabberd_config:is_file_readable(Path) of
+            PathS = binary_to_list(Path),
+	    case ejabberd_config:is_file_readable(PathS) of
 		true -> true;
-		false -> {false, Path}
+		false -> {false, PathS}
 	    end
     end.
 
@@ -505,7 +553,7 @@ socket_error(Reason, PortIP, Module, SockOpts, Port, IPS) ->
 		      "IP address not available: " ++ IPS;
 		  eaddrinuse ->
 		      "IP address and port number already used: "
-			  ++IPS++" "++integer_to_list(Port);
+			  ++binary_to_list(IPS)++" "++integer_to_list(Port);
 		  _ ->
 		      format_error(Reason)
 	      end,
@@ -520,3 +568,104 @@ format_error(Reason) ->
 	ReasonStr ->
 	    ReasonStr
     end.
+
+-define(IS_CHAR(C), (is_integer(C) and (C >= 0) and (C =< 255))).
+-define(IS_UINT(U), (is_integer(U) and (U >= 0) and (U =< 65535))).
+-define(IS_PORT(P), (is_integer(P) and (P > 0) and (P =< 65535))).
+-define(IS_TRANSPORT(T), ((T == tcp) or (T == udp))).
+
+transform_option({{Port, IP, Transport}, Mod, Opts}) ->
+    IPStr = if is_tuple(IP) ->
+                    list_to_binary(inet_parse:ntoa(IP));
+               true ->
+                    IP
+            end,
+    Opts1 = lists:map(
+              fun({ip, IPT}) when is_tuple(IPT) ->
+                      {ip, list_to_binary(inet_parse:ntoa(IP))};
+                 (tls) -> {tls, true};
+                 (ssl) -> {tls, true};
+                 (zlib) -> {zlib, true};
+                 (starttls) -> {starttls, true};
+                 (starttls_required) -> {starttls_required, true};
+                 (Opt) -> Opt
+              end, Opts),
+    Opts2 = lists:foldl(
+              fun(Opt, Acc) ->
+                      try
+                          Mod:transform_listen_option(Opt, Acc)
+                      catch error:undef ->
+                              [Opt|Acc]
+                      end
+              end, [], Opts1),
+    TransportOpt = if Transport == tcp -> [];
+                      true -> [{transport, Transport}]
+                   end,
+    IPOpt = if IPStr == <<"0.0.0.0">> -> [];
+               true -> [{ip, IPStr}]
+            end,
+    IPOpt ++ TransportOpt ++ [{port, Port}, {module, Mod} | Opts2];
+transform_option({{Port, Transport}, Mod, Opts})
+  when ?IS_TRANSPORT(Transport) ->
+    transform_option({{Port, {0,0,0,0}, Transport}, Mod, Opts});
+transform_option({{Port, IP}, Mod, Opts}) ->
+    transform_option({{Port, IP, tcp}, Mod, Opts});
+transform_option({Port, Mod, Opts}) ->
+    transform_option({{Port, {0,0,0,0}, tcp}, Mod, Opts});
+transform_option(Opt) ->
+    Opt.
+
+transform_options(Opts) ->
+    lists:foldl(fun transform_options/2, [], Opts).
+
+transform_options({listen, LOpts}, Opts) ->
+    [{listen, lists:map(fun transform_option/1, LOpts)} | Opts];
+transform_options(Opt, Opts) ->
+    [Opt|Opts].
+
+-type transport() :: udp | tcp.
+-type port_ip_transport() :: inet:port_number() |
+                             {inet:port_number(), transport()} |
+                             {inet:port_number(), inet:ip_address()} |
+                             {inet:port_number(), inet:ip_address(),
+                              transport()}.
+-spec validate_cfg(list()) -> [{port_ip_transport(), module(), list()}].
+
+validate_cfg(L) ->
+    lists:map(
+      fun(LOpts) ->
+              lists:foldl(
+                fun({port, Port}, {{_, IP, T}, Mod, Opts}) ->
+                        true = ?IS_PORT(Port),
+                        {{Port, IP, T}, Mod, Opts};
+                   ({ip, IP}, {{Port, _, T}, Mod, Opts}) ->
+                        {{Port, prepare_ip(IP), T}, Mod, Opts};
+                   ({transport, T}, {{Port, IP, _}, Mod, Opts}) ->
+                        true = ?IS_TRANSPORT(T),
+                        {{Port, IP, T}, Mod, Opts};
+                   ({module, Mod}, {Port, _, Opts}) ->
+                        {Port, prepare_mod(Mod), Opts};
+                   (Opt, {Port, Mod, Opts}) ->
+                        {Port, Mod, [Opt|Opts]}
+                end, {{5222, {0,0,0,0}, tcp}, ejabberd_c2s, []}, LOpts)
+      end, L).
+
+prepare_ip({A, B, C, D} = IP)
+  when ?IS_CHAR(A) and ?IS_CHAR(B) and ?IS_CHAR(C) and ?IS_CHAR(D) ->
+    IP;
+prepare_ip({A, B, C, D, E, F, G, H} = IP)
+  when ?IS_UINT(A) and ?IS_UINT(B) and ?IS_UINT(C) and ?IS_UINT(D)
+       and ?IS_UINT(E) and ?IS_UINT(F) and ?IS_UINT(G) and ?IS_UINT(H) ->
+    IP;
+prepare_ip(IP) when is_list(IP) ->
+    {ok, Addr} = inet_parse:address(IP),
+    Addr;
+prepare_ip(IP) when is_binary(IP) ->
+    prepare_ip(binary_to_list(IP)).
+
+prepare_mod(ejabberd_sip) ->
+    prepare_mod(sip);
+prepare_mod(sip) ->
+    esip_socket;
+prepare_mod(Mod) when is_atom(Mod) ->
+    Mod.
