@@ -5,7 +5,7 @@
 %%% Created :  1 Nov 2006 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2011   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2014   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
 %%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 %%% General Public License for more details.
 %%%
-%%% You should have received a copy of the GNU General Public License
-%%% along with this program; if not, write to the Free Software
-%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-%%% 02111-1307 USA
+%%% You should have received a copy of the GNU General Public License along
+%%% with this program; if not, write to the Free Software Foundation, Inc.,
+%%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
 %%%----------------------------------------------------------------------
 
@@ -40,11 +39,7 @@
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
 	 terminate/2, code_change/3]).
 
--ifdef(SSL40).
 -define(PG2, pg2).
--else.
--define(PG2, pg2_backport).
--endif.
 
 -record(state, {}).
 
@@ -60,20 +55,20 @@ start_link() ->
 
 join(Name) ->
     PG = {?MODULE, Name},
-    ?PG2:create(PG),
-    ?PG2:join(PG, whereis(?MODULE)).
+    pg2:create(PG),
+    pg2:join(PG, whereis(?MODULE)).
 
 leave(Name) ->
     PG = {?MODULE, Name},
-    ?PG2:leave(PG, whereis(?MODULE)).
+    pg2:leave(PG, whereis(?MODULE)).
 
 get_members(Name) ->
     PG = {?MODULE, Name},
-    [node(P) || P <- ?PG2:get_members(PG)].
+    [node(P) || P <- pg2:get_members(PG)].
 
 get_closest_node(Name) ->
     PG = {?MODULE, Name},
-    node(?PG2:get_closest_pid(PG)).
+    node(pg2:get_closest_pid(PG)).
 
 %%====================================================================
 %% gen_server callbacks
@@ -88,7 +83,12 @@ get_closest_node(Name) ->
 %%--------------------------------------------------------------------
 init([]) ->
     {FE, BE} =
-	case ejabberd_config:get_local_option(node_type) of
+	case ejabberd_config:get_option(
+               node_type,
+               fun(frontend) -> frontend;
+                  (backend) -> backend;
+                  (generic) -> generic
+               end, generic) of
 	    frontend ->
 		{true, false};
 	    backend ->
