@@ -5,7 +5,7 @@
 %%% Created : 27 Jan 2006 by Alexey Shchepin <alexey@process-one.net>
 %%%
 %%%
-%%% ejabberd, Copyright (C) 2002-2011   ProcessOne
+%%% ejabberd, Copyright (C) 2002-2014   ProcessOne
 %%%
 %%% This program is free software; you can redistribute it and/or
 %%% modify it under the terms of the GNU General Public License as
@@ -17,10 +17,9 @@
 %%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 %%% General Public License for more details.
 %%%
-%%% You should have received a copy of the GNU General Public License
-%%% along with this program; if not, write to the Free Software
-%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-%%% 02111-1307 USA
+%%% You should have received a copy of the GNU General Public License along
+%%% with this program; if not, write to the Free Software Foundation, Inc.,
+%%% 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 %%%
 %%%-------------------------------------------------------------------
 
@@ -31,6 +30,7 @@
 -export([update/0, update/1, update_info/0]).
 
 -include("ejabberd.hrl").
+-include("logger.hrl").
 
 %%====================================================================
 %% API
@@ -69,13 +69,9 @@ update(ModulesToUpdate) ->
 
 %% OTP R14B03 and older provided release_handler_1:eval_script/3
 %% But OTP R14B04 and newer provide release_handler_1:eval_script/5
+%% Dialyzer reports a call to missing function; don't worry.
 eval_script(Script, Apps, LibDirs) ->
-    case lists:member({eval_script, 5}, release_handler_1:module_info(exports)) of
-	true ->
-	    release_handler_1:eval_script(Script, Apps, LibDirs, [], []);
-	false ->
-	    release_handler_1:eval_script(Script, Apps, LibDirs)
-    end.
+    release_handler_1:eval_script(Script, Apps, LibDirs, [], []).
 
 %% Get information about the modified modules
 update_info() ->
@@ -141,21 +137,26 @@ build_script(Dir, UpdatedBeams) ->
 	release_handler_1:check_script(
 	  LowLevelScript,
 	  [{ejabberd, "", filename:join(Dir, "..")}]),
-    case Check of
-	ok -> 
-	    ?DEBUG("script: ~p~n", [Script]),
-	    ?DEBUG("low level script: ~p~n", [LowLevelScript]),
-	    ?DEBUG("check: ~p~n", [Check]);
-	{ok, []} ->
-	    ?DEBUG("script: ~p~n", [Script]),
-	    ?DEBUG("low level script: ~p~n", [LowLevelScript]),
-	    ?DEBUG("check: ~p~n", [Check]);
-	_ ->
-	    ?ERROR_MSG("script: ~p~n", [Script]),
-	    ?ERROR_MSG("low level script: ~p~n", [LowLevelScript]),
-	    ?ERROR_MSG("check: ~p~n", [Check])
-    end,
-    {Script, LowLevelScript, Check}.
+    Check1 = case Check of
+                 ok ->
+                     %% This clause is for OTP R14B03 and older.
+                     %% Newer Dialyzer reports a never match pattern; don't worry.
+                     ?DEBUG("script: ~p~n", [Script]),
+                     ?DEBUG("low level script: ~p~n", [LowLevelScript]),
+                     ?DEBUG("check: ~p~n", [Check]),
+                     ok;
+                 {ok, []} ->
+                     ?DEBUG("script: ~p~n", [Script]),
+                     ?DEBUG("low level script: ~p~n", [LowLevelScript]),
+                     ?DEBUG("check: ~p~n", [Check]),
+                     ok;
+                 _ ->
+                     ?ERROR_MSG("script: ~p~n", [Script]),
+                     ?ERROR_MSG("low level script: ~p~n", [LowLevelScript]),
+                     ?ERROR_MSG("check: ~p~n", [Check]),
+                     error
+             end,
+    {Script, LowLevelScript, Check1}.
 
 %% Copied from Erlang/OTP file: lib/sasl/src/systools.hrl
 -record(application, 
